/*

lcdcpu: cpu status functions
Copyleft 1997 Andrew J. Kroll (ag784@freenet.buffalo.edu)

*/

#include "lcd.h"

void init_cpustat (void)
{
	FILE *nf;
	char s_dv [12];
	nf=fopen ("/proc/stat", "rt");
	fscanf (nf, "%s %llu %llu %llu %llu",
		s_dv, &last_user, &last_nice, &last_sys, &last_idle);
	fclose (nf);
} /* init_proc -- PS */

float cpus=1;

void cpustat (int *xc_user, int *xc_nice, int *xc_sys, int *xc_idle)
{
	unsigned long long tmp_user, tmp_nice, tmp_sys, tmp_idle;
	float user_per, nice_per, sys_per, idle_per;
	int xc_total, tcpu;
	FILE *nf;
	char s_dv [12];
	nf=fopen ("/proc/stat", "rt");
	fscanf (nf, "%s %llu %llu %llu %llu", s_dv, &tmp_user, &tmp_nice, &tmp_sys, &tmp_idle);
	fclose (nf);
	*xc_user = tmp_user - last_user;
	*xc_nice = tmp_nice - last_nice;
	*xc_sys = tmp_sys - last_sys;
	*xc_idle = tmp_idle - last_idle;
	tcpu=0;
	if (*xc_user<0) { tcpu=tcpu - *xc_user; *xc_user=0; }
	if (*xc_nice<0) { tcpu=tcpu - *xc_nice; *xc_nice=0; }
	if (*xc_sys <0) { tcpu=tcpu - *xc_sys ; *xc_sys =0; }
	if (*xc_idle<0) { tcpu=tcpu - *xc_idle; *xc_idle=0; }

/*
SMP and the headache:

CPU states: 89.6% user, 109.6% system, 160.9% nice,  0.0% idle
usr + sys  = total cpu power used
89.6+109.6 = 199.2

total - potential = idle
199.2 - 200       = 0.8

nice = 160.9
idle = 0.8

nice  + idle = usrsys
160.9 + 0.8  = 161.7

total - usrsys = not nice
199.2 - 161.7  = 37.5

OK, so how much of the not nice is user, and how much is system?
Whoever wrote the /proc/stat code originally was smoking some bad crack!
Each CPU should have it's own entry, plus a combined entry.

Guess I gotta add another proc patch, I'll bitch to the boys about it first.

*/

	xc_total = *xc_user + *xc_nice + *xc_sys + *xc_idle + tcpu;
	last_user = tmp_user;
	last_nice = tmp_nice;
	last_sys = tmp_sys;
	last_idle = tmp_idle;

	user_per = ((float) *xc_user / (xc_total * 100));
	nice_per = ((float) *xc_nice / (xc_total * 100));
	sys_per = ((float) *xc_sys / (xc_total * 100));
	idle_per = ((float) *xc_idle / (xc_total * 100));
	
#ifdef DEBUG
	printf("user:%5.2f%%(%d) nice:%5.2f%%(%d) sys:%5.2f%%(%d) idle:%5.2f%%(%d) total:%5.2f%%(%d)\n", user_per, *xc_user, nice_per, *xc_nice, sys_per, *xc_sys, idle_per, *xc_idle, user_per + nice_per + sys_per + idle_per, xc_total)
#endif
} /* cpustat -- PS */


/* cpu usage graphs, staggered, inside of char cells */
void draw_cpugraph (int fd, int pass)
{

	int i, c_user=0, c_nice=0, c_sys=0, c_idle=0;
	int amount=0, lastpixel, remainingpixels;
	char picture[81];
	                    
	cpustat(&c_user, &c_nice, &c_sys, &c_idle);
	
	amount=(c_user/((double)c_idle+(double)c_user+
					(double)c_nice+(double)c_sys))*11;
	

/* 
Since cpu and some other graph is printed on the same cells (swap/memory),
we ignore the position fields, and only draw out out pixels.
Positioning is handled with the graph command.
*/


	/* user */
	lastpixel=amount;
	sprintf(picture, "%5.5s", "     ");
	if (amount > 0) {
		for(i=0;i<5;i++) {
			picture[i]='X';
			amount=amount-1;
			if(amount==0) break;
		}
	}
	dump_char_data (fd, picture, 48);

	sprintf(picture,"%5.5s","     ");
	if (amount > 0) {
		for(i=0;i<5;i++) {
			picture[i]='X';
			amount=amount-1;
			if(amount==0) break;
		}
	}
	dump_char_data (fd, picture, 56);
	
	/* system */
	amount=((double)c_sys/((double)c_idle+(double)c_user+
					(double)c_nice+(double)c_sys))*11;
	
	remainingpixels=amount+lastpixel;
	sprintf(picture,"%5.5s","     ");
	if (amount > 0) {
		for(i=lastpixel;i<5;i++) {
			picture[i]='X';
			amount=amount-1;
			if(amount==0) break;
		}
	}
	dump_char_data (fd, picture, 49);

	sprintf(picture,"%5.5s","     ");
	lastpixel=lastpixel-5; if(lastpixel<0) lastpixel=0;
	if (amount > 0) {
		for(i=lastpixel;i<5;i++) {
			picture[i]='X';
			amount=amount-1;
			if(amount==0) break;
		}
	}
	dump_char_data (fd, picture, 57);
	
	/* nice */
	amount=((double)c_nice/((double)c_idle+(double)c_user+
    					(double)c_nice+(double)c_sys))*11;

	sprintf(picture,"%5.5s","     ");
	if (amount > 0) {
		for(i=remainingpixels;i<5;i++) {
			picture[i]='X';
			amount=amount-1;
			if(amount==0) break;
		}
	}
	dump_char_data (fd, picture, 50);

	sprintf(picture,"%5.5s","     ");
	remainingpixels=remainingpixels-5; if(remainingpixels<0) remainingpixels=0;
	if (amount > 0) {
		for(i=remainingpixels;i<5;i++) {
			picture[i]='X';
			amount=amount-1;
			if(amount==0) break;
		}
	}
	dump_char_data (fd, picture, 58);

}
