/*

lcdroot: LCD root communication functions
Copyleft 1997 Andrew J. Kroll (ag784@freenet.buffalo.edu)

*/

#define LCDROOT

#include "lcd.h"

char graphArray[6] = {0x00, 0x10, 0x18, 0x1c, 0x1e, 0x1f};

void lcd_pos (int fd, int pos)
{
int status;
struct LCD_DATA_TYPE lcd;

status=ioctl (fd, LCD_SETCONTROL);
if (status == -1) {
perror ("lcd");
exit (1);
}

if ((pos<0)||(pos>0x40+40)) {
printf ("lcd: position out of range\n");
exit (1);
}

lcd.text[0]=128+pos;
lcd.text[1]=0;

	status = write (fd, &lcd, LCD_DATASIZE);
	if (status != LCD_DATASIZE) {
		perror ("lcd");
		exit (1);
	}
}

void lcd_write (int fd, char *str)
{
	struct LCD_DATA_TYPE lcd;
	int status;

	strcpy (lcd.text, str);
	ioctl (fd, LCD_SETDATA);
	status = write (fd, &lcd, LCD_DATASIZE);
	if (status != LCD_DATASIZE) {
		perror ("lcd");
		exit (1);
	}
}

void lcd_init (int fd)
{
	int status;
	struct LCD_DATA_TYPE lcd;

	status=ioctl (fd, LCD_SETCONTROL);
	if (status == -1) {
		perror ("lcd");
		exit (1);
	}

	strcpy (lcd.text, LCD_INITSTRING);
	status = write (fd, &lcd, LCD_DATASIZE);
	if (status != LCD_DATASIZE) {
		perror ("lcd");
		exit (1);
	}
	usleep (1000);
}

void lcd_open (int *fd)
{
	*fd = open ("/dev/lcd0", O_WRONLY);
	if (*fd < 0) {
		perror ("lcd");
		exit (1);
	}
	lcd_init (*fd);
}

void lcd_clear (int fd)
{
	int status;
	struct LCD_DATA_TYPE lcd;

	status=ioctl (fd, LCD_SETCONTROL);
	if (status == -1) {
		perror ("lcd");
		exit (1);
	}

	strcpy (lcd.text, LCD_CLEARSTRING);
	status = write (fd, &lcd, LCD_DATASIZE);
	if (status != LCD_DATASIZE) {
		perror ("lcd");
		exit (1);
	}
}

void lcd_gRAM (int fd, int add)
{
	int status;
	char datm[2];
	struct LCD_DATA_TYPE lcd;

	datm[1]=0;
	datm[0]=add+0x40;
	status=ioctl (fd, LCD_SETCONTROL);
	if (status == -1) {
		perror ("lcd");
		exit (1);
	}

	strcpy (lcd.text, datm);
	status = write (fd, &lcd, LCD_DATASIZE);
	if (status != LCD_DATASIZE) {
		perror ("lcd");
		exit (1);
	}
}


void dump_char_data (int fd, char *bfr, int rpos)
{
int key, gpos, mypos, ln;
char text[2];
/* notes...

the data is simply a string of ' ' and 'X' and defines the "shape"
each shape "byte" is 5 characters long.

*/
	lcd_gRAM(fd,rpos); /* first ram position */
	/* sprintf(mbfr,"%s",bfr); */
	ln=strlen(bfr)/5;
	for(mypos=0;mypos<ln;mypos++) {
		/* update char gen RAM */	
		key=0;
		for(gpos=0;gpos<5;gpos++) {
			key = (key << 1) | (((int) bfr[gpos+5*mypos] & 8) >> 3);
		}
	sprintf(text,"%c",(char) key);
	lcd_write (fd, text);
	}
}


void lcd_draw_bar (int fd, int pixels, int pos)
{
	char lna[2]="a", lnb[2]="a";

	if (pixels < 5) {
		lna[0] = graphArray[pixels];
                lnb[0] = graphArray[0];
        } else {
                lna[0] = graphArray[5];
                lnb[0] = graphArray[pixels - 5];
        }
        lcd_gRAM(fd, pos);
        lcd_write (fd, lna);
        lcd_gRAM(fd, pos + 8);
        lcd_write (fd, lnb);
}
